<?php // phpcs:disable PSR1.Files.SideEffects.FoundWithSymbols

/*
 * Plugin Name:          WPify Woo
 * Description:          Custom functionality for WooCommerce
 * Version:              3.3.12
 * Requires PHP:         7.3.0
 * Requires at least:    5.3
 * Author:               WPify s.r.o.
 * Author URI:           https://www.wpify.io/
 * License:              GPLv2 or later
 * License URI:          https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:          wpify-woo
 * Domain Path:          /languages
 * WC requires at least: 4.5
 * WC tested up to:      5.1
*/

use WpifyWoo\Plugin;
use WpifyWooDeps\DI;
use WpifyWooDeps\DI\Definition\Helper\AutowireDefinitionHelper;
use WpifyWooDeps\Wpify\Core\Container;
use WpifyWooDeps\Wpify\Core\WebpackManifest;

if ( ! defined( 'WPIFY_WOO_MIN_PHP_VERSION' ) ) {
	define( 'WPIFY_WOO_MIN_PHP_VERSION', '7.3.0' );
}

/**
 * Singleton instance function. We will not use a global at all as that defeats the purpose of a singleton
 * and is a bad design overall
 * @SuppressWarnings(PHPMD.StaticAccess)
 * @return WpifyWoo\Plugin
 * @throws Exception
 */
function wpify_woo(): Plugin {
	return wpify_woo_container()->get( Plugin::class );
}

/**
 * This container singleton enables you to setup unit testing by passing an environment file to map classes in Dice
 *
 * @param string $env
 *
 * @return DI\Container
 * @throws Exception
 */
function wpify_woo_container(): DI\Container {
	static $container;

	if ( empty( $container ) ) {
		$wpify_container = Container::getInstance();
		$container       = $wpify_container->get_container( 'wpify_woo' );
		if ( ! $container ) {
			$container = $wpify_container->add_container(
					'wpify_woo',
					array(
							Plugin::class                               => new AutowireDefinitionHelper( Plugin::class ),
							WebpackManifest::class                      => ( new AutowireDefinitionHelper() )
									->constructor( 'build/assets-manifest.json', 'wpify-woo~' ),
					)
			);
		}
	}

	return $container;
}

/**
 * Init function shortcut
 */
function wpify_woo_init() {
	wpify_woo()->init();
}

/**
 * Activate function shortcut
 */
function wpify_woo_activate( $network_wide ) {
	register_uninstall_hook( __FILE__, 'wpify_woo_uninstall' );
	wpify_woo()->init();
	wpify_woo()->activate( $network_wide );
}

/**
 * Deactivate function shortcut
 */
function wpify_woo_deactivate( $network_wide ) {
	wpify_woo()->deactivate( $network_wide );
}

/**
 * Uninstall function shortcut
 */
function wpify_woo_uninstall() {
	wpify_woo()->uninstall();
}

/**
 * Error for older php
 */
function wpify_woo_php_upgrade_notice() {
	$info = get_plugin_data( __FILE__ ); ?>
	<div class="error notice">
		<p>
			<?php printf( _e( 'Opps! %1$s requires a minimum PHP version of %2$s. Your current version is: %3$s. Please contact your host to upgrade.', 'dogsie' ), $info['Name'], WPIFY_WOO_MIN_PHP_VERSION, PHP_VERSION ); ?>
		</p>
	</div>
	<?php
}

/**
 * Error if vendors autoload is missing
 */
function wpify_woo_php_vendor_missing() {
	$info = get_plugin_data( __FILE__ );
	?>
	<div class="error notice">
		<p><?php printf( __( 'Opps! %s is corrupted it seems, please re-install the plugin.', 'wpify-woo' ), $info['Name'] ); ?></p>
	</div>
	<?php
}

/**
 * WooCommerce not active notice
 */
function wpify_woo_woocommerce_not_active() {
	?>
	<div class="error notice">
		<p><?php _e( 'This plugin requires WooCommerce. Please install and activate it first.', 'wpify-woo' ); ?></p>
	</div>
	<?php
}

/**
 * Load plugin textdomain.
 */
add_action( 'init', 'wpify_woo_load_textdomain' );
function wpify_woo_load_textdomain() {
	load_plugin_textdomain( 'wpify-woo', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}

function wpify_woo_require_compatibility() {
	$wpify_plugins = array();
	$wpify_check   = array(
			'wpify-woo-sodexo/wpify-woo-sodexo.php'                               => '1.5.0',
			'wpify-woo-fakturoid/wpify-woo-fakturoid.php'                         => '1.2.0',
			'wpify-woo-benefit-plus/wpify-woo-benefit-plus.php'                   => '1.4.0',
			'wpify-woo-podani-online/wpify-woo-podani-online.php'                 => '1.2.1',
			'wpify-woo-phone-validation/wpify-woo-phone-validation.php'           => '1.3.1',
			'wpify-woo-zbozi-conversions/wpify-woo-zbozi-conversions.php'         => '1.2.1',
			'wpify-woo-conditional-shipping/wpify-woo-conditional-shipping.php'   => '1.0.7',
			'wpify-woo-vivnetworks-affiliate/wpify-woo-vivnetworks-affiliate.php' => '1.2.1',
			'wpify-woo-comgate/wpify-woo-comgate.php'                             => '1.1.0',
	);

	foreach ( glob( WP_PLUGIN_DIR . '/*/*.php' ) as $file ) {
		if ( strpos( $file, 'wpify-woo' ) !== false ) {
			$file_data = get_file_data( $file, array(
					'name'    => 'Plugin Name',
					'version' => 'Version',
			) );

			foreach ( $wpify_check as $plugin => $version ) {
				if ( strpos( $file, $plugin ) ) {
					$wpify_plugins[] = array(
							'name'                  => $file_data['name'],
							'current_version'       => $file_data['version'],
							'compatibility_version' => $version,
							'plugin'                => $plugin,
							'file'                  => $file,
							'compatible'            => version_compare( $file_data['version'], $version, '>' ),
					);
				}
			}
		}
	}

	$incompatible = array_filter( $wpify_plugins, function ( $plugin ) {
		return ! $plugin['compatible'];
	} );

	if ( count( $incompatible ) ) {
		require_once __DIR__ . '/compat/vendor/autoload.php';
		require_once __DIR__ . '/compat/vendor/php-di/php-di/src/functions.php';
	}
}

/*
 * We want to use a fairly modern php version, feel free to increase the minimum requirement
 */
if ( version_compare( PHP_VERSION, WPIFY_WOO_MIN_PHP_VERSION ) < 0 ) {
	add_action( 'admin_notices', 'wpify_woo_php_upgrade_notice' );
} elseif ( ! in_array( 'woocommerce/woocommerce.php', (array) get_option( 'active_plugins', array() ), true ) ) {
	add_action( 'admin_notices', 'wpify_woo_woocommerce_not_active' );
} else {
	if ( file_exists( __DIR__ . '/deps/scoper-autoload.php' ) ) {
		include_once __DIR__ . '/deps/scoper-autoload.php';
		include_once __DIR__ . '/lib/PacketeraSDK/vendor/autoload.php';
		include_once __DIR__ . '/vendor/autoload.php';

		wpify_woo_require_compatibility();
		add_action( 'plugins_loaded', 'wpify_woo_init', 11 );
		register_activation_hook( __FILE__, 'wpify_woo_activate' );
		register_deactivation_hook( __FILE__, 'wpify_woo_deactivate' );
	} else {
		add_action( 'admin_notices', 'wpify_woo_php_vendor_missing' );
	}
}
